<?php

defined('ABSPATH') or die('This page may not be accessed directly.');

class Cuttr
{
    const API_URL = 'https://cuttr.me/api?key={key}&url={url}';

    const SAVE_SETTINGS_NONCE_KEY = 'cuttr_page_nonce';
    const SAVE_SETTINGS_NONCE_ACTION = 'cuttr_page';

    const CUTTR_META_LINK = 'cuttr_link';
    const CUTTR_META_LINK_PERMALINK = 'cuttr_permalink';

    public static function init()
    {
        $cuttr = new self();

        if (CuttrUtils::canModifySettings()) {
            add_action('admin_menu', array(__CLASS__, 'addAdminPage'));
        }

        add_action('add_meta_boxes', array(__CLASS__, 'addMetaBoxes'));
        add_action('transition_post_status', array(__CLASS__, 'onTransitionPostStatus'), 10, 3);
        add_action('rest_api_init', array(__CLASS__, 'updateRestApiResponse'));


        add_filter('post_row_actions', array(__CLASS__, 'addLinkPostRowAction'), 10, 2);
        add_filter('manage_posts_columns', array(__CLASS__, 'addCuttrColumn'));
        add_action('manage_posts_custom_column', array(__CLASS__, 'addDataCuttrColumn'));

        return $cuttr;
    }

    public static function updateRestApiResponse()
    {
        register_rest_field(
            'post',
            'link',
            array(
                'get_callback' => function ($post) {
                    $settings = CuttrSettings::getSettings();
                    $shortLink = self::getShortLink($post['id']);

                    return $settings['replace_link_meta_publish'] && !empty($shortLink) ? $shortLink : $post['link'];
                },
            )
        );
    }

    public static function getShortLink($postId)
    {
        return get_post_meta($postId, Cuttr::CUTTR_META_LINK, true);
    }

    public static function addAdminPage()
    {
        add_menu_page(
            'Cuttr',
            'Cuttr',
            'manage_options',
            'cuttr-plugin',
            array(__CLASS__, 'loadView')
        );

        self::saveSettingsForm();
    }

    public static function saveSettingsForm()
    {
        if (array_key_exists('api_key', $_POST)) {
            check_admin_referer(self::SAVE_SETTINGS_NONCE_ACTION, self::SAVE_SETTINGS_NONCE_KEY);
            self::saveSettings($_POST);
        }
    }

    public static function saveSettings($newSettings)
    {
        if (!CuttrUtils::canModifySettings()) {
            return;
        }

        $settings = CuttrSettings::getSettings();
        $stringSettings = array(
            'api_key',
            'replace_link_meta_publish',
        );

        self::saveStringSettings($settings, $newSettings, $stringSettings);
        CuttrSettings::saveSettings($settings);
    }

    public static function saveStringSettings(&$settings, &$newSettings, $stringSettings)
    {
        foreach ($stringSettings as $stringSetting) {
            $value = sanitize_text_field(isset($newSettings[$stringSetting]) ? $newSettings[$stringSetting] : null);

            if ('api_key' === $stringSetting && CuttrUtils::maskApiKey($settings[$stringSetting]) === $value) {
                continue;
            }

            $settings[$stringSetting] = $value;
        }
    }

    public static function loadView()
    {
        require_once __DIR__.'/../views/index.php';
    }

    public static function addMetaBoxes()
    {
        global $post;

        if ('publish' === $post->post_status || 'future' === $post->post_status) {
            add_meta_box(
                'cuttr-link',
                'Cuttr',
                array(__CLASS__, 'addMetaBoxLinkOnPostHtmlView'),
                'post',
                'side',
                'high'
            );
        }
    }

    public static function addMetaBoxLinkOnPostHtmlView($post)
    {
        $cuttrLink = self::getShortLink($post->ID);
        if (!empty($cuttrLink) && ('publish' === $post->post_status || 'future' === $post->post_status)) {
            echo sprintf('<p>%s : <a href="%2$s" rel="noopener" target="_blank">%2$s</a></p>', __('View post'), $cuttrLink);
        }
    }

    public static function onTransitionPostStatus($newStatus, $oldStatus, $post)
    {
        if ($post->post_type === 'wdslp-wds-log' || $oldStatus === 'trash' || ('publish' !== $newStatus && 'future' !== $newStatus)) {
            return;
        }

        self::generateUrl($post);
    }

    public static function generateUrl($post)
    {
        $settings = CuttrSettings::getSettings();
        if (empty($settings['api_key'])) {
            return;
        }

        $postId = $post->ID;
        $permalink = get_permalink($postId);
        $permalinkHashed = md5($permalink);
        if ($permalinkHashed === self::getShortLinkHash($postId)) {
            return;
        }

        $response = @file_get_contents(str_replace(
            ['{key}', '{url}'],
            [$settings['api_key'], urlencode(filter_var($permalink, FILTER_SANITIZE_URL))],
            self::API_URL
        ));
        $data = json_decode($response, true);

        if (empty($data)) {
            return;
        }

        if (!empty($data['short'])) {
            update_post_meta($postId, self::CUTTR_META_LINK, $data['short']);
            update_post_meta($postId, self::CUTTR_META_LINK_PERMALINK, $permalinkHashed);
        }
    }

    public static function getShortLinkHash($postId)
    {
        return get_post_meta($postId, Cuttr::CUTTR_META_LINK_PERMALINK, true);
    }

    public static function addLinkPostRowAction($actions, $post)
    {
        $cuttrLink = self::getShortLink($post->ID);
        if (!empty($cuttrLink) && ('publish' === $post->post_status || 'future' === $post->post_status)) {
            $actions['cuttr_link'] = sprintf('<a href="%s" rel="noopener" target="_blank">%s</a>', $cuttrLink, __('Cuttr', 'cuttr'));
        }

        return $actions;
    }

    public static function addCuttrColumn($columns)
    {
        return array_merge($columns, array('cuttr' => __('Cuttr')));
    }

    public static function addDataCuttrColumn($name)
    {
        global $post;

        $cuttrLink = self::getShortLink($post->ID);
        if ('cuttr' === $name && !empty($cuttrLink) && ('publish' === $post->post_status || 'future' === $post->post_status)) {
            echo sprintf('<a href="%1$s" rel="noopener" target="_blank">%1$s</a>', $cuttrLink);
        }
    }
}
